using System;
using System.Numerics;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BasicPlayerMovement : MonoBehaviour
{
    public float Speed = 4f;
    public float HitForce = 50f;
    public bool UseKeyboard;

   // public bool HoldingMatching => _currentSocks.Count == 2 && _currentSocks[0].SockMaterial == _currentSocks[1].SockMaterial;
    public int PlayerIndex;
    private Rigidbody _rigidbody;
    private SockBin _sockBinPlacement;
    bool isPunching = false;
    bool canBeHit = true;
    bool canUseLeftHand = true;
    bool canUseRightHand = true;
    
    [SerializeField] ParticleSystem hitParticles;

    [SerializeField] Collider punchBox;
    [SerializeField] SockHoldPosition leftSock;
    [SerializeField] SockHoldPosition rightSock;
    [SerializeField] Animator playerAnimator;
    
    SockBehaviour highlightedSock;

    private void Awake()
    {
        _rigidbody = GetComponent<Rigidbody>();
    }

    private void Update()
    {
        var ext = PlayerIndex > 0 && !UseKeyboard ? PlayerIndex.ToString() : "";

        var x = Input.GetAxis("Horizontal"+ ext);
        var z = Input.GetAxis("Vertical"+ ext);
        //Debug.Log($"X: {x}, Z: {z}");
        var pickUpLeft = Input.GetAxis($"InteractA{ext}");
        var pickUpRight = Input.GetAxis($"InteractB{ext}");
        var punch = Input.GetAxis($"Punch{ext}");
        Debug.Log($"Robot: Punch{ext}");


        float pickUpLeftAlt = 0f;
        float pickUpRightAlt = 0f;

        if(!string.IsNullOrEmpty(ext))
        {
            pickUpLeftAlt = Input.GetAxis($"InteractA{ext}Alt");
            pickUpRightAlt = Input.GetAxis($"InteractB{ext}Alt");
        }

        float movespeed = Math.Abs(x) + Math.Abs(z);

        if(canBeHit)
        {
            playerAnimator.SetFloat("movespeed", movespeed);
            
            var movementVector = new UnityEngine.Vector3(x, 0f, z) * Time.deltaTime * Speed;

            _rigidbody.velocity = movementVector;
            transform.LookAt(transform.position + movementVector);
        }
        else
        {
            playerAnimator.SetFloat("movespeed", 0f);
            //_rigidbody.velocity = new UnityEngine.Vector3(0f, 0f, 0f);
        }

        var isPressingLeft = pickUpLeft > 0 || pickUpLeftAlt > 0;
        var isPressingRight = pickUpRight > 0 || pickUpRightAlt > 0;

        var isHoldingSockLeft = leftSock.GetSock() != null;
        var isHoldingSockRight = rightSock.GetSock() != null;

        var sockHighlighted = highlightedSock != null;

        
        if (isPressingLeft && canBeHit)
        {
            if (!isHoldingSockLeft && sockHighlighted && highlightedSock.CanPickUp)
            {
                pickUpSock(highlightedSock, leftSock);
            }
        }
        else
        {
            if (isHoldingSockLeft)
            {
                if (_sockBinPlacement != null && leftSock.GetSock() != null && rightSock.GetSock() != null)
                {
                    _sockBinPlacement.CheckSockResult(leftSock.GetSock(), rightSock.GetSock(), PlayerIndex-1);
                    Destroy(leftSock.GetSock().gameObject);
                    Destroy(rightSock.GetSock().gameObject);
                    leftSock.AssignSock(null);
                    rightSock.AssignSock(null);
                }
                else
                {
                    dropSock(leftSock);
                }
            }
        }
        if (isPressingRight && canBeHit)
        {
            if (!isHoldingSockRight && sockHighlighted && highlightedSock.CanPickUp)
            {
                pickUpSock(highlightedSock, rightSock);
            }
        }
        else
        {
            if (isHoldingSockRight)
            {
                if (_sockBinPlacement != null && rightSock.GetSock() != null && leftSock.GetSock() != null)
                {
                    _sockBinPlacement.CheckSockResult(leftSock.GetSock(), rightSock.GetSock(), PlayerIndex-1);
                    Destroy(rightSock.GetSock().gameObject);
                    Destroy(leftSock.GetSock().gameObject);
                    rightSock.AssignSock(null);
                    leftSock.AssignSock(null);
                }
                else
                {
                    dropSock(rightSock);
                }
            }
        }

        if (punch > 0 && !isPunching && !(leftSock.GetSock() != null && rightSock.GetSock() != null))
        {
            Punch();
        }

    }


    void pickUpSock(SockBehaviour sb, SockHoldPosition holdPosition)
    {
        sb.transform.parent = holdPosition.transform;
        sb.transform.localPosition = UnityEngine.Vector3.zero;
        sb.PickedUp(PlayerIndex);
        holdPosition.AssignSock(sb);
    }

    void dropSock(SockHoldPosition holdPosition)
    {
        SockBehaviour sb;
        sb = holdPosition.GetSock();
        if(sb != null)
        {
            holdPosition.AssignSock(null);
            sb.Dropped(PlayerIndex, _rigidbody.velocity);
            sb.HideSockOutline();
        }
    }

    // highlighting and hitting
    void OnTriggerEnter(Collider other)
    {
        if(other.gameObject.GetComponentInParent<SockBehaviour>() != null)
        {
            if (leftSock.GetSock() != null && rightSock.GetSock() != null)
            {
                return;
            }
            var sb = other.gameObject.GetComponentInParent<SockBehaviour>();
            if(!sb.CanPickUp)
            {
                return;
            }
            sb.ShowSockOutline(PlayerIndex);
            highlightedSock = sb;
        }
        else if(other.gameObject.GetComponent<BasicPlayerMovement>() != null)
        {
            playerHit(other.gameObject.GetComponent<BasicPlayerMovement>());
        }

    }

    // highlighting
    void OnTriggerExit(Collider other)
    {
       var sb = other.gameObject.GetComponentInParent<SockBehaviour>();
       if(sb == null)
       {
           return;
       }
        sb.HideSockOutline();
        if(sb == highlightedSock)
        {
            highlightedSock = null;
        }

    }

    void Punch()
    {
        punchBox.enabled = true;
        isPunching = true;
        playerAnimator.SetTrigger("punch");
        StartCoroutine(punchCooldown());
    }

    IEnumerator punchCooldown()
    {
        yield return new WaitForSeconds(0.3f);
        punchBox.enabled = false;
        yield return new WaitForSeconds(0.15f);
        isPunching = false;
    }

    IEnumerator leftHandCooldown()
    {
        yield return new WaitForSeconds(0.2f);
        canUseLeftHand = true;
    }

    IEnumerator rightHandCooldown()
    {
        yield return new WaitForSeconds(0.2f);
        canUseRightHand = true;
    }

    public void SetSockPlacement(SockBin bin)
    {
        _sockBinPlacement = bin;
    }

    public void playerHit(BasicPlayerMovement player)
    {
        playerAnimator.SetTrigger("prone");
        hitParticles.Play();
        var direction = transform.position - player.transform.position;
        _rigidbody.AddForceAtPosition(direction * HitForce, player.transform.position, ForceMode.Impulse);
        canBeHit = false;
        StartCoroutine(ProneCoolDown());
    }

    IEnumerator ProneCoolDown()
    {
        yield return new WaitForSeconds(2f);
        canBeHit = true;
    }

    public UnityEngine.Vector3 getDirectionVector()
    {
        return _rigidbody.velocity;
    }

    

}
