﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using System.Linq;

public class EndGame : MonoBehaviour
{
    public float MaxY;
    public List<GameObject> Players;
    public List<Image> EndGameCanvasScores;

    private void Awake()
    {
        foreach (var c in EndGameCanvasScores)
        {
            c.gameObject.SetActive(false);
        }
    }

    public void ShowGameOver(int[] scores)
    {
        for (var i=0;i<scores.Length; i++)
        {
            var y = GetYPos(scores[i], scores);
            //Players[i].transform.position += Vector3.up * y;
            StartCoroutine(DoPlayerRaise(Players[i], y, y));
        }
        StartCoroutine(WaitToShowScores(scores, 2.25f));
    }

    private IEnumerator DoPlayerRaise(GameObject go, float yPos, float total)
    {
        yield return new WaitForSeconds(1f);
        var start = go.transform.localPosition;
        var end = new Vector3(go.transform.localPosition.x, go.transform.localPosition.y + yPos, go.transform.localPosition.z);
        var elapsed = 0f;

        while (elapsed <= total)
        {
            go.transform.localPosition = Vector3.Lerp(start, end, elapsed / total);
            elapsed += Time.deltaTime;
            yield return null;
        }
        go.transform.localPosition = end;
    }

    private IEnumerator WaitToShowScores(int[] scores, float delay)
    {
        yield return new WaitForSeconds(delay);
        for (var i = 0; i < EndGameCanvasScores.Count; i++)
        {
            EndGameCanvasScores[i].GetComponentInChildren<Text>().text = scores[i].ToString();
            EndGameCanvasScores[i].gameObject.SetActive(true);
        }
    }

    private float GetYPos(int score, int[] allScores)
    {
        //var playersBeat = allScores.Count(s => score >= s);
        //var y = (MaxY / allScores.Length) * playersBeat;
        //return y;
        var max = allScores.Max();
        var y = ((float)score / max) * MaxY;
        return Mathf.Clamp(y, 0f, MaxY);
    }
}
