﻿using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameManager : MonoBehaviour
{
    public static GameManager Instance;
    [SerializeField] private GameObject _startScreen;
    [SerializeField] private SockSpawner _sockSpawner;
    [SerializeField] private Animation _washingMachineAnimation;
    [SerializeField] private EndGame _endGame;
    [SerializeField] private GameObject _endGameCamera;

    public static event Action TriggerSockTornado;
    public static event Action StopSockTornado;

    private List<SockBehaviour> _socks = new List<SockBehaviour>();

    private bool _gamePlaying;

    private int[] _scores;

    private void Start()
    {
        _scores = new int[4];// number of players
        SockBehaviour.sockOutOfBounds += SockOutOfBounds;
    }

    private void Update()
    {
        if (Input.anyKeyDown && _startScreen.activeSelf)
        {
            _startScreen.gameObject.SetActive(false);
            StartCoroutine(CreateSocks());
        }
        if (_gamePlaying && (_socks.Count == 0 || (_socks.Count > 0 &&_socks.All(s => s == null))))
        {
            _gamePlaying = false;
            _endGame.gameObject.SetActive(true);
            _endGameCamera.SetActive(true);
            _endGame.ShowGameOver(_scores);
        }
        if(Input.GetAxis("start") > 0 && _gamePlaying)
        {
            UnityEngine.SceneManagement.SceneManager.LoadScene(0);
        }

        if(Input.GetKeyDown(KeyCode.Escape))
        {
            Application.Quit();
        }
    }
    private IEnumerator CreateSocks()
    {
        yield return _sockSpawner.CreateSocks();
        _socks = _sockSpawner.CreatedSocks;
        StartGame();
    }

    public void StartGame()
    {
        _gamePlaying = true;
        StartCoroutine(WaitForAnimToStop());
    }

    private IEnumerator WaitForAnimToStop()
    {
        yield return new WaitForSeconds(3f);
        var time = _washingMachineAnimation[_washingMachineAnimation.clip.name].time;
        while (time < 0.975f)
        {
            yield return null;
        }
        _washingMachineAnimation[_washingMachineAnimation.clip.name].time = 0f;
        _washingMachineAnimation[_washingMachineAnimation.clip.name].speed = 0f;

    }

    public void RemoveSock(SockBehaviour sock)
    {
        if (_socks.Contains(sock))
        {
            _socks.Remove(sock);
            Destroy(sock.gameObject);   
        }
    }

    public void PauseGame()
    {
        _gamePlaying = false;
    }

    void SockOutOfBounds(Material mat)
    {
        StartCoroutine(RecreateSock(mat));
    }

    public IEnumerator RecreateSock(Material mat)
    {
        yield return _sockSpawner.CreateSock(mat);
        _socks.Add(_sockSpawner.CreatedSocks[0]);
    }

    public void OnPointScored(int playerNumber)
    {
        _scores[playerNumber]++;
    }
}
