﻿using System;
using System.Numerics;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SockBehaviour : MonoBehaviour
{
    public bool CanPickUp;
    public Transform originalParent;

    [SerializeField] cakeslice.Outline[] sockOutlines;
    public Material SockMaterial { get; private set; }

    bool hasConstraints = true;

    [SerializeField] Collider[] sockCollider; 

    private Rigidbody _rigidbody;
    [SerializeField] int repositioningForce;
    [SerializeField] private UnityEngine.Vector3 sockBounds;
    [SerializeField] private UnityEngine.Vector3 sockBoundsOffset;
    public static event Action<Material> sockOutOfBounds;

    bool isSockTornado = false;


    private void Awake()
    {
        _rigidbody = GetComponent<Rigidbody>();
        CanPickUp = true;
        StartCoroutine(DroppingConstraints());
        originalParent = transform.parent;

        // GameManager.TriggerSockTornado += StartSockTornado;
        // GameManager.StopSockTornado += StopSockTornado;
    }

    IEnumerator DroppingConstraints()
    {
        yield return new WaitForSeconds(3.5f);
        hasConstraints = false;
    }

    public void PickedUp(int playerNumber)
    {
        CanPickUp = false;

        _rigidbody.useGravity = false;
        _rigidbody.constraints = RigidbodyConstraints.FreezeAll;

        foreach(Collider c in sockCollider)
        {
            c.enabled = false;
        }
    }

    public void Dropped(int playerNumber, UnityEngine.Vector3 force)
    {
        transform.SetParent(originalParent);
        _rigidbody.useGravity = true;
        _rigidbody.constraints = RigidbodyConstraints.None;

        _rigidbody.AddForce(force);
        foreach(Collider c in sockCollider)
        {
            c.enabled = true;
        }
		CanPickUp = true;
		//StartCoroutine(WaitBeforeMakingInteractable());
    }

    //private IEnumerator WaitBeforeMakingInteractable()
    //{
    //    yield return new WaitForSeconds(.35f);
       
    //}

    public void ShowSockOutline(int playerNumber)
    {   
        foreach(cakeslice.Outline o in sockOutlines)
        {
            o.color = playerNumber - 1;
            o.enabled = true;
        }
    }

    public void HideSockOutline()
    {
        foreach(cakeslice.Outline o in sockOutlines)
        {
            o.enabled = false;
        }
    }

    private void Update()
    {
        // _rigidbody.useGravity = CanPickUp;
        // _rigidbody.constraints = CanPickUp ? RigidbodyConstraints.None : RigidbodyConstraints.FreezeAll;

        if(CanPickUp && hasConstraints)
        {
            if(transform.position.x > (sockBounds.x + sockBoundsOffset.x) || transform.position.x < (sockBounds.x - sockBoundsOffset.x)) 
            {
                _rigidbody.AddForce(new UnityEngine.Vector3((transform.position.x > 0) ? -repositioningForce : repositioningForce, 0, 0), ForceMode.Acceleration);
            }
            if(transform.position.z > (sockBounds.z + sockBoundsOffset.x) || transform.position.z < (sockBounds.z - sockBoundsOffset.z))
            {
                _rigidbody.AddForce(new UnityEngine.Vector3(0, 0,(transform.position.z > 0) ? -repositioningForce : repositioningForce), ForceMode.Acceleration);
            }
        }

        if(transform.position.y < -5)
        {
            sockOutOfBounds(SockMaterial);
            Destroy(gameObject);
        }

    }

    public void SetMaterial(Material m)
    {
        SockMaterial = m;
        GetComponentInChildren<SkinnedMeshRenderer>().material = SockMaterial;
    }

}
